@@ -1,6 +1,6 @@
 /* ----------------------------------------------------------------------------
    libconfig - A structured configuration file parsing library
-   Copyright (C) 2005  Mark A Lindner
+   Copyright (C) 2005-2007  Mark A Lindner
  
    This file is part of libconfig.
     
@@ -533,6 +533,23 @@ LIBCONFIG_API void config_init(config_t *config)
 
 // ---------------------------------------------------------------------------
 
+LIBCONFIG_API void config_set_auto_convert(config_t *config, int flag)
+{
+  if(flag)
+    config->flags |= CONFIG_OPTION_AUTOCONVERT;
+  else
+    config->flags &= ~CONFIG_OPTION_AUTOCONVERT;
+}
+
+// ---------------------------------------------------------------------------
+
+LIBCONFIG_API int config_get_auto_convert(const config_t *config)
+{
+  return((config->flags & CONFIG_OPTION_AUTOCONVERT) != 0);
+}
+
+// ---------------------------------------------------------------------------
+
 static config_setting_t *config_setting_create(config_setting_t *parent,
                                                const char *name, int type)
 {
@@ -550,6 +567,7 @@ static config_setting_t *config_setting_create(config_setting_t *parent,
   setting->type = type;
   setting->config = parent->config;
   setting->hook = NULL;
+  setting->line = 0;
 
   list = parent->value.list;
   
@@ -565,27 +583,68 @@ static config_setting_t *config_setting_create(config_setting_t *parent,
 
 LIBCONFIG_API long config_setting_get_int(const config_setting_t *setting)
 {
-  return((setting->type == CONFIG_TYPE_INT) ? setting->value.ival : 0);
+  switch(setting->type)
+  {
+    case CONFIG_TYPE_INT:
+      return(setting->value.ival);
+      
+    case CONFIG_TYPE_FLOAT:
+      if((setting->config->flags & CONFIG_OPTION_AUTOCONVERT) != 0)
+        return((long)(setting->value.fval));
+      else
+        /* fall through */;
+      
+    default:
+      return(0);
+  }
 }
 
 // ---------------------------------------------------------------------------
 
 LIBCONFIG_API int config_setting_set_int(config_setting_t *setting, long value)
 {
-  if(setting->type == CONFIG_TYPE_NONE)
-    setting->type = CONFIG_TYPE_INT;
-  else if(setting->type != CONFIG_TYPE_INT)
-    return(CONFIG_FALSE);
-  
-  setting->value.ival = value;
-  return(CONFIG_TRUE);
+  switch(setting->type)
+  {
+    case CONFIG_TYPE_NONE:
+      setting->type = CONFIG_TYPE_INT;
+      /* fall through */
+
+    case CONFIG_TYPE_INT:
+      setting->value.ival = value;
+      return(CONFIG_TRUE);
+
+    case CONFIG_TYPE_FLOAT:
+      if(config_get_auto_convert(setting->config))
+      {
+        setting->value.fval = (float)value;
+        return(CONFIG_TRUE);
+      }
+      else
+        return(CONFIG_FALSE);
+
+    default:
+      return(CONFIG_FALSE);
+  }
 }
 
 // ---------------------------------------------------------------------------
 
 LIBCONFIG_API double config_setting_get_float(const config_setting_t *setting)
 {
-  return((setting->type == CONFIG_TYPE_FLOAT) ? setting->value.fval : 0.0);
+  switch(setting->type)
+  {
+    case CONFIG_TYPE_FLOAT:
+      return(setting->value.fval);
+
+    case CONFIG_TYPE_INT:
+      if(config_get_auto_convert(setting->config))
+        return((double)(setting->value.ival));
+      else
+        /* fall through */;
+
+    default:
+      return(0.0);
+  }
 }
 
 // ---------------------------------------------------------------------------
@@ -593,13 +652,28 @@ LIBCONFIG_API double config_setting_get_float(const config_setting_t *setting)
 LIBCONFIG_API int config_setting_set_float(config_setting_t *setting,
                                            double value)
 {
-  if(setting->type == CONFIG_TYPE_NONE)
-    setting->type = CONFIG_TYPE_FLOAT;
-  else if(setting->type != CONFIG_TYPE_FLOAT)
-    return(CONFIG_FALSE);
-  
-  setting->value.fval = value;
-  return(CONFIG_TRUE);
+  switch(setting->type)
+  {
+    case CONFIG_TYPE_NONE:
+      setting->type = CONFIG_TYPE_FLOAT;
+      /* fall through */
+
+    case CONFIG_TYPE_FLOAT:
+      setting->value.fval = value;
+      return(CONFIG_TRUE);
+
+    case CONFIG_TYPE_INT:
+      if((setting->config->flags & CONFIG_OPTION_AUTOCONVERT) != 0)
+      {
+        setting->value.ival = (long)value;
+        return(CONFIG_TRUE);
+      }
+      else
+        return(CONFIG_FALSE);
+
+    default:
+      return(CONFIG_FALSE);
+  }
 }
 
 // ---------------------------------------------------------------------------
@@ -650,7 +724,7 @@ LIBCONFIG_API int config_setting_set_string(config_setting_t *setting,
 // ---------------------------------------------------------------------------
 
 LIBCONFIG_API int config_setting_set_format(config_setting_t *setting,
-                                            int format)
+                                            short format)
 {
   if((setting->type != CONFIG_TYPE_INT)
      || ((format != CONFIG_FORMAT_DEFAULT) && (format != CONFIG_FORMAT_HEX)))
@@ -663,7 +737,7 @@ LIBCONFIG_API int config_setting_set_format(config_setting_t *setting,
 
 // ---------------------------------------------------------------------------
 
-LIBCONFIG_API int config_setting_get_format(config_setting_t *setting)
+LIBCONFIG_API short config_setting_get_format(config_setting_t *setting)
 {
   return(setting->format);
 }
@@ -751,13 +825,7 @@ LIBCONFIG_API long config_setting_get_int_elem(const config_setting_t *vector,
 {
   const config_setting_t *element = config_setting_get_elem(vector, index);
 
-  if(! element)
-    return(0);
-  
-  if(element->type != CONFIG_TYPE_INT)
-    return(0);
-  
-  return(element->value.ival);
+  return(element ? config_setting_get_int(element) : 0);
 }
 
 // ---------------------------------------------------------------------------
@@ -798,13 +866,7 @@ LIBCONFIG_API double config_setting_get_float_elem(
 {
   config_setting_t *element = config_setting_get_elem(vector, index);
 
-  if(! element)
-    return(0.0);
-  
-  if(element->type != CONFIG_TYPE_FLOAT)
-    return(0.0);
-
-  return(element->value.fval);
+  return(element ? config_setting_get_float(element) : 0.0);
 }
 
 // ---------------------------------------------------------------------------
