@@ -1,6 +1,6 @@
 /* ----------------------------------------------------------------------------
    libconfig - A structured configuration file parsing library
-   Copyright (C) 2005  Mark A Lindner
+   Copyright (C) 2005-2007  Mark A Lindner
  
    This file is part of libconfig.
     
@@ -28,14 +28,16 @@ extern "C" {
 #endif /* __cplusplus */
 
 #if defined(_WIN32) || defined(__WIN32__) || defined(WIN32)
-#if defined(DLL_EXPORT) || defined(LIBCONFIG_EXPORTS)
-#define LIBCONFIG_API __declspec(dllexport)
-#else // ! DLL_EXPORT
-#define LIBCONFIG_API __declspec(dllimport)
-#endif // DLL_EXPORT
-#else // ! WIN32
+#if defined(LIBCONFIG_STATIC)
 #define LIBCONFIG_API
-#endif // WIN32
+#elif defined(LIBCONFIG_EXPORTS)
+#define LIBCONFIG_API __declspec(dllexport)
+#else /* ! LIBCONFIG_EXPORTS */
+#define LIBCONFIG_API __declspec(dllimport)
+#endif /* LIBCONFIG_STATIC */
+#else /* ! WIN32 */
+#define LIBCONFIG_API
+#endif /* WIN32 */
 
 #include <stdio.h>
 
@@ -51,6 +53,8 @@ extern "C" {
 #define CONFIG_FORMAT_DEFAULT  0
 #define CONFIG_FORMAT_HEX      1
 
+#define CONFIG_OPTION_AUTOCONVERT 0x01
+  
 #define CONFIG_TRUE  (1)
 #define CONFIG_FALSE (0)
 
@@ -66,12 +70,13 @@ typedef union config_value_t
 typedef struct config_setting_t
 {
   char *name;
-  int type;
-  int format;
+  short type;
+  short format;
   config_value_t value;
   struct config_setting_t *parent;
   struct config_t *config;
   void *hook;
+  unsigned int line;
 } config_setting_t;
 
 typedef struct config_list_t
@@ -85,6 +90,7 @@ typedef struct config_t
 {
   config_setting_t *root;
   void (*destructor)(void *);
+  int flags;
   const char *error_text;
   int error_line;
 } config_t;
@@ -92,6 +98,9 @@ typedef struct config_t
 extern LIBCONFIG_API int config_read(config_t *config, FILE *stream);
 extern LIBCONFIG_API void config_write(const config_t *config, FILE *stream);
 
+extern LIBCONFIG_API void config_set_auto_convert(config_t *config, int flag);
+extern LIBCONFIG_API int config_get_auto_convert(const config_t *config);
+
 extern LIBCONFIG_API int config_read_file(config_t *config,
                                           const char *filename);
 extern LIBCONFIG_API int config_write_file(config_t *config,
@@ -122,8 +131,8 @@ extern LIBCONFIG_API int config_setting_set_string(config_setting_t *setting,
                                                    const char *value);
 
 extern LIBCONFIG_API int config_setting_set_format(config_setting_t *setting,
-                                                   int format);
-extern LIBCONFIG_API int config_setting_get_format(config_setting_t *setting);
+                                                   short format);
+extern LIBCONFIG_API short config_setting_get_format(config_setting_t *setting);
 
 extern LIBCONFIG_API long config_setting_get_int_elem(
   const config_setting_t *setting, int index);
@@ -146,7 +155,27 @@ extern LIBCONFIG_API config_setting_t *config_setting_set_string_elem(
 #define /* int */ config_setting_type(/* const config_setting_t * */ S) \
   ((S)->type)
 
-#define /* const char */ config_setting_name(/* const config_setting_t * */ S) \
+#define /* int */ config_setting_is_group(/* const config_setting_t * */ S) \
+  ((S)->type == CONFIG_TYPE_GROUP)
+#define /* int */ config_setting_is_array(/* const config_setting_t * */ S) \
+  ((S)->type == CONFIG_TYPE_ARRAY)
+#define /* int */ config_setting_is_list(/* const config_setting_t * */ S) \
+  ((S)->type == CONFIG_TYPE_LIST)
+
+#define /* int */ config_setting_is_aggregate( \
+  /* const config_setting_t * */ S)                                     \
+  (((S)->type == CONFIG_TYPE_GROUP) || ((S)->type == CONFIG_TYPE_LIST)  \
+   || ((S)->type == CONFIG_TYPE_ARRAY))
+  
+#define /* int */ config_setting_is_number(/* const config_setting_t * */ S) \
+  (((S)->type == CONFIG_TYPE_INT) || ((S)->type == CONFIG_TYPE_FLOAT))
+  
+#define /* int */ config_setting_is_scalar(/* const config_setting_t * */ S) \
+  (((S)->type == CONFIG_TYPE_BOOL) || ((S)->type == CONFIG_TYPE_STRING) \
+   || config_setting_is_number(S))
+
+#define /* const char * */ config_setting_name( \
+  /* const config_setting_t * */ S)             \
   ((S)->name)
 
 extern LIBCONFIG_API int config_setting_length(
@@ -178,15 +207,20 @@ extern LIBCONFIG_API int config_lookup_bool(const config_t *config,
 extern LIBCONFIG_API const char *config_lookup_string(const config_t *config,
                                                       const char *path);
 
-#define /* config_setting_t * */ config_root_setting(/* const config_t * */ C) \
+#define /* config_setting_t * */ config_root_setting( \
+  /* const config_t * */ C)                           \
   ((C)->root)
-
-#define /* const char * */ config_error_text(/* const config_t * */ C)    \
+  
+#define /* unsigned short */ config_setting_source_line( \
+  /* const config_t */ C)                                \
+  ((C)->line)
+  
+#define /* const char * */ config_error_text(/* const config_t * */ C)  \
   ((C)->error_text)
-
-#define /* int */ config_error_line(/* const config_t * */ C)     \
+  
+#define /* int */ config_error_line(/* const config_t * */ C)   \
   ((C)->error_line)
-
+  
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
