@@ -25,7 +25,7 @@ int main()
     /* Add setting "foo" */
     cout << "add setting \"foo\"..";
     Setting &root = cfg.getRoot();
-    Setting &foo  = root.add("foo",TypeInt);
+    Setting &foo  = root.add("foo", Setting::TypeInt);
     foo = 1234;
     cout << "ok" << endl;
 
